/***************************************************************************
  FILE: TLAgentFake.java, TomaszMajor-10May98
  PURPOSE: use with TServerFake
  $Id: TLAgentFake.java,v 1.10 1999/11/13 00:57:14 toma Exp $
  Component: Plugin, Interfaces
***************************************************************************/


package webwisdom.tango.fake;


import java.net.*;
import java.io.*;
import java.awt.*;
import java.util.Vector;
import java.util.Hashtable;
import java.util.StringTokenizer;

import webwisdom.tango.RegParam;
import webwisdom.tango.TangoException;
import webwisdom.tango.TLAgent;
import webwisdom.tango.TLListener;
import webwisdom.tango.messages.AppEventMessage;
import webwisdom.tango.messages.CAppAnswerEventMessage;
import webwisdom.tango.consts.Const;
import webwisdom.tango.consts.CAConsts;

import netscape.security.PrivilegeManager;


class TLAgentFakeReceiver extends Thread
{
    private static final String CL="TLAgentFakeReceiver";
    private static final String CL2="FTA";


    DataInputStream inStream;
    TLAgentFake client;


    public TLAgentFakeReceiver(DataInputStream is,TLAgentFake tc)
    {
      //System.out.println(CL+".TLAgentFakeReceiver("+is+","+tc+")");
      inStream=is;
      client=tc;
    }// TLAgentFakeReceiver ________________________________________________


    public void run()
    {
      //System.out.println(CL+".run()");
      try
      {
	do
	{
	  byte type=inStream.readByte();
	  if(type==TServerFake.DATA)
	  {
	    int packetLen=inStream.readInt();
	    byte[] packetData=new byte[packetLen];
	    inStream.readFully(packetData);
	    client.receive(packetData);
	    //System.out.println(CL2+": < DATA["+packetLen+"]");
	  }
	  else if(type==TServerFake.MASTER)
	  {
	    System.out.println(CL2+": < MASTER");
	    client.setMaster(true);
	  }
	  else if(type==TServerFake.SLAVE)
	  {
	    System.out.println(CL2+": < SLAVE");
	    client.setMaster(false);
	  }
	}
	while(true);
      }
      catch(EOFException e)
      {
	System.err.println(CL2+": Server disconnected!");
      }
      catch(IOException e)
      {
	System.err.println(CL2+": "+e);
	e.printStackTrace();
      }
    }// run ________________________________________________________________


    public String toString()
    {
	return CL+"[]";
    }// toString ___________________________________________________________


}// TLAgentFakeReceiver ////////////////////////////////////////////////////




class CAppAnswerEventMessageFake extends CAppAnswerEventMessage
{
    private static final String CL="CAppAnswerEventMessageFake";


    private byte type;
    private TLAgentFake agent;


    CAppAnswerEventMessageFake(AppEventMessage m,TLAgentFake a)
    {
      super(new byte[1]);
      type=m.getData()[0];
      agent=a;
    }// CAppAnswerEventMessageFake _________________________________________


    public byte getAnswerType()
    {
      return type;
    }
    

    public boolean isMaster()
    {
      return agent.isMaster.booleanValue();
    }


    public String getMastername()
    {
      return agent.masterName;
    } 
    

    public String getUsername()
    {
      return agent.userName;
    }
    

    public Vector getParticipants()
    {
      return agent.participantNames;
    }


    public Hashtable getPartipuids()
    {
      return agent.partipuids;
    }


    public Hashtable getRoles()
    {
      return agent.roles;
    }


    public Vector getLocalApps()
    {
      return new Vector();
    }


    public Rectangle getGeometry()
    {
      return new Rectangle(-1,-1,600,400);
    }


    public String toString()
    {
      return CL+"[]";
    }// toString ___________________________________________________________


}// CAppAnswerEventMessageFake /////////////////////////////////////////////




/**This class can be used instead of TangoAgent
   for browserless testing of Tango applets with FakeTangoServer.
   *@see FakeTangoServer
   *@author Tomasz Major*/
public class TLAgentFake implements TLAgent
{
    private static final String CL="TLAgentFake";
    private static final String CL2="FTA";


    public static String PARAM_FAKESERVER="tango.fakeServer";


    private TLListener lowListener=null;


    private Socket socket;
    private DataOutputStream outStream;
    private DataInputStream inStream;

    private TLAgentFakeReceiver receiverThread;


    private int applicationId=-1;


    private Object isMasterLock=new Object();
    Boolean isMaster=null;
    String userName="user";
    Vector participantNames=new Vector();
    String masterName="master";//new String(userName);
    Hashtable partipuids=new Hashtable();
    Hashtable roles=new Hashtable();


    /**Creates TLAgent which connects to TServerFake.
     *@param serverHost,serverPort fake tango server
     *@exception TangoException if cannot connect to the server*/
    public TLAgentFake(String[] args)
      throws TangoException
    {
      String param=RegParam.getParameter(args,PARAM_FAKESERVER);
      if(param==null)
        throw new TangoException("missing parameter: "+PARAM_FAKESERVER);

      String host;
      int port;
      try
      {
	StringTokenizer st=new StringTokenizer(param,":");

	host=st.nextToken();
	port=Integer.valueOf(st.nextToken()).intValue();

	//System.out.println(CL+".TLAgentFake(): host="+host+",port="+port);
      }
      catch(NumberFormatException e)
      {
	throw new TangoException("format error: "+param);
      }
      init(host,port);
    }// TLAgentFake ________________________________________________________


    /**Creates TLAgent which connects to TServerFake.
     *@param serverHost,serverPort fake tango server
     *@exception TangoException if cannot connect to the server*/
    public TLAgentFake(String serverHost,int serverPort)
      throws TangoException
    {
      init(serverHost,serverPort);
    }// TLAgentFake ________________________________________________________


    /**Main part of the constructor.
     *@param serverHost,serverPort fake tango server
     *@exception TangoException if cannot connect to the server*/
    private void init(String serverHost,int serverPort)
      throws TangoException
    {
      //System.out.println(CL+".init("+serverHost+","+serverPort+")");
      netscape.security.PrivilegeManager.enablePrivilege("UniversalConnect");
      try
      {
	socket=new Socket(serverHost,serverPort);
	outStream=new DataOutputStream(socket.getOutputStream());
	inStream=new DataInputStream(socket.getInputStream());
	System.out.println(CL2+": connected to: "+socketToString(socket)+
			   " (localPort="+socket.getLocalPort()+")");
      }
      catch(IOException e)
      {
	throw new TangoException(serverHost+":"+serverPort+" unreachable");
      }
      //System.out.println(CL+": socket="+socket+",inStream="+inStream+",outStream="+outStream);


      setSessionAttributes("ben","jerry",null);


      receiverThread=new TLAgentFakeReceiver(inStream,this);
      //receiverThread.setPriority(Thread.MIN_PRIORITY);
      receiverThread.start();
      (Thread.currentThread()).setPriority(Thread.MIN_PRIORITY);
    }// init _______________________________________________________________


    /**
     *@param u this user name
     *@param m name of the session master
     *@param us names of other users in the session*/
    public void setSessionAttributes(String u,String m,String[] us)
    {
      //System.out.println(CL+".setSessionAttributes("+u+","+m+","+us+")");

      userName=u;
      masterName=(m==null)?new String(u):m;
      //masterName="maestro";

      boolean uAdded=false,mAdded=false;
      Vector userNames=new Vector();
      if(us!=null)
      {
	for(int i=0;i<us.length;i++)
	{
	  String ui=us[i];
	  if(ui.equals(userName))
	    uAdded=true;
	  else if(ui.equals(masterName))
	    mAdded=true;
	  userNames.addElement(ui);
	}
      }
      if(!uAdded)
	userNames.addElement(u);
      if(!mAdded)
	userNames.addElement(masterName);

      //applicationId=(int)userName.hashCode();
      //System.out.println(CL2+": id="+applicationId+",userName="+userName+",masterName="+masterName);
      applicationId=(int)System.currentTimeMillis();//-Long.MAX_VALUE+3600);
      System.out.println(CL2+": id="+applicationId);

      for(int i=participantNames.size()-1;i>=0;i--)
      {
	partipuids.put((String)participantNames.elementAt(i),new Integer(i));
	roles.put(new Integer(i),new String("student"));
      }
    }// setSessionAttributes _______________________________________________


    public void exit()
    {
      receiverThread.stop();
      try
      {
	inStream.close();
	outStream.close();
	socket.close();
      }
      catch(IOException e)
      {
      }
    }// exit _______________________________________________________________


    public void setTLListener(TLListener l)
    {
      lowListener=l;
    }// setTLListener ______________________________________________________


    public void send(int i[],AppEventMessage m)
    {
    }// send _______________________________________________________________

  
    public void send(AppEventMessage m)
    {
      try
      {
	if(m.getType()==Const.CONTROL)
	{
	  if((m.getData()[0]==CAConsts.ISMASTER)&&(isMaster==null))
	  {
	    System.out.println(CL2+": waiting for master...");
	    synchronized(isMasterLock)
	    {
	      try
	      {
		isMasterLock.wait();
	      }
	      catch(InterruptedException e)
	      {
		e.printStackTrace();
	      }
	    }
	  }
	  if(lowListener!=null)
	    lowListener.receive(new CAppAnswerEventMessageFake(m,this));
	}
	else
	{
	  synchronized(outStream)
	  {
	    byte[] b=m.getData();
	    outStream.writeInt(b.length);
	    outStream.write(b);
	    outStream.flush();
	    //System.out.println(CL+".send(byte["+b.length+"])");
	  }
	}
      }
      catch(IOException e)
      {
	System.err.println(CL+".send(): "+e);
	e.printStackTrace();
      }
    }// send _______________________________________________________________


    /*interface for TLAgentFakeReceiver*/
    void receive(byte[] b)
    {
      if(lowListener!=null)
	lowListener.receive(new AppEventMessage(Const.DATA,b));
    }// receive ____________________________________________________________


    /*interface for TLAgentFakeReceiver*/
    void setMaster(boolean isM)
    {
      //System.out.println(CL+".setMaster("+isM+")");
      if(isMaster==null)
      {
	System.out.println(CL2+": finished waiting for master.");
	synchronized(isMasterLock)
	{
	  isMaster=new Boolean(isM);
	  isMasterLock.notifyAll();
	}
      }
      else
      {
	isMaster=new Boolean(isM);
      }
      if(isM)
	masterName=userName;
      if(lowListener!=null)
      {
	byte[] b=new byte[2];
	b[0]=CAConsts.ISMASTER;
	b[1]=isM?CAConsts.YES:CAConsts.NO;
	CAppAnswerEventMessage m=new CAppAnswerEventMessage(b);
	//System.out.println(CL+".setMaster(): "+m);
	lowListener.receive(m);
      }
    }// receive ____________________________________________________________


    public int getSessionId()
    {
      return 1;
    }// getSessionId _______________________________________________________


    public int getApplicationId()
    {
      //if(applicationId==-1)
      //applicationId=(int)System.currentTimeMillis();
      return applicationId;
    }// getApplicationId ___________________________________________________


    /*utility*/
    public static String socketToString(Socket s)
    {
      return "["+s.getInetAddress().getHostName()+":"+s.getPort()+"]";
    }// socketToString _____________________________________________________


    public String toString()
    {
      return CL+"["+socketToString(socket)+"]";
    }// toString ___________________________________________________________


}// TLAgentFake ////////////////////////////////////////////////////////////


