// $Id: AppSetUpdateMessage.java,v 1.3 1999/12/21 21:57:04 beca Exp $
/*
File: AppSetMessage.java
Component: Server, Plugin
Date		Author		Changes
Dec 20 99	Lukasz Beca	Created
*/


package webwisdom.tango.messages;


import webwisdom.tango.messages.Message;
import webwisdom.tango.consts.Const;
import webwisdom.tango.test.Log;
import java.io.*;  
import java.util.*;


/**
 * A class representing APPSET message
 * 
 *@author Lukasz Beca
 *@see Central Server - Local Demon protocol
 **/
public class AppSetUpdateMessage extends Message
{
    private static final String CL="AppSetUpdateMessage";
    
    private Hashtable setTable;


/**
   * Creates a new SUPD message
   *@param PARAMparam update type: Const.ADDUPDATE, Const.REMOVEUPDATE, Const.MASTERUPDATE, Const.LOGINUPDATE, Const.LOGOUTUPDATE
   *@param SIDparam session number
   *@param ATpar application type
   *@param UIDparam user number
   *@param namepar user name
   **/

  public AppSetUpdateMessage(Hashtable setTableParam) {
   
   setTable = (Hashtable) setTableParam.clone();
   
   type = Const.APPSETUPDATE;
  }

 /**
   * Creates a new SUPD message, information is read from the input stream
   *@param instream  input stream
   **/

  AppSetUpdateMessage(DataInputStream instream) throws IOException {
             
    ObjectInputStream ois;
    ByteArrayInputStream ais; 
    int len;
    byte table[];
    
    try {
            
        
        len = instream.readInt();
        table = new byte[len];
        instream.readFully(table);
        ais = new ByteArrayInputStream(table);
        ois = new ObjectInputStream(ais);
        setTable = (Hashtable) ois.readObject();
        type = Const.APPSETUPDATE;             
        
    } catch(ClassNotFoundException e) {
        
        Log.err.println(e.getMessage(), 2);   
    }    
  }

 /**
   * Writes SUPD message to the output stream
   *@param outstream  output stream
   **/


  void send(DataOutputStream outstream) throws IOException {
     
    ByteArrayOutputStream aos = new ByteArrayOutputStream();
    ObjectOutputStream oos = new ObjectOutputStream(aos); 
    oos.writeObject(setTable);    
    
    byte table[] = aos.toByteArray();
     
    outstream.writeInt(Const.APPSETUPDATE);     
    outstream.writeInt(table.length);
    outstream.write(table);    
    outstream.flush();          
   
  }
   

 /**
   * Returns session number
   **/        

  public Hashtable getSetTable() {
   
    return setTable;
  }
  
  public String toString()
    {
      return CL+"[setTable="+setTable+"]";
    }// toString __________________________________________________________


}// SupdMessage ///////////////////////////////////////////////////////////


