// $Id: Watcher.java,v 1.7 2000/02/15 21:52:50 beca Exp $
/*
  File: Watcher.java
  Component: Plugin
  Date		Author		Changes
  Jun 28 99	Lukasz Beca	Created  
*/

package webwisdom.tango.threads;

import java.net.*;
import java.io.*;
import webwisdom.tango.structures.ControlApplication;
import webwisdom.tango.structures.MessageQueue;
import webwisdom.tango.test.Log;
import webwisdom.tango.messages.WatcherMessage;
import webwisdom.tango.messages.Message;
import webwisdom.tango.messages.WatchNet;
import webwisdom.tango.consts.Const;



public class Watcher {
 
    private ControlApplication ca;
    private WatchThreadIn input;
    private WatchThreadOut output;
    private boolean reconnectEnabled;
    private boolean inTango;
    private String serverHost;
    private int port;
    private String url;
    
    public static final byte UNDEFINED = 0;
    public static final byte USER_CONNECTED = 1;
    public static final byte USER_DISCONNECTED = 2;
    public static final byte PING = 3;
     
    public Watcher(ControlApplication control) {
        
        Socket s;        
        ca = control;  
        reconnectEnabled = true;
        inTango = false;
        serverHost = null; 
        port = 0;
        url= null;
        
        try {            
	   		if(Const.inNetscape()) 
			{
				netscape.security.PrivilegeManager.enablePrivilege("30Capabilities");
			}
			if(Const.inMSIE())
			{
				com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.SYSTEM); 
			}
            s = new Socket("localhost", 4403);        
            output = new WatchThreadOut(s);
            input = new WatchThreadIn(s, output, this);                        
        } catch (Exception e) {
            
            Log.out.println("Watcher: PeopleWatch not installed", 4);
            s = null;
            output = null;
            input = null;            
        }
     }
    
    public void userConnected(String serverHost, int port, String url) {
        
      WatcherMessage m;
      String serverString = serverHost + ":" + port;
      byte serverData[];
      byte urlData[];
      byte combinedData[];
      int length1, length2, totalLength;
      int i;
      
      // init state data
      this.serverHost = serverHost;
      this.port = port;
      this.url = url;
      
      inTango = true;
	//retrieving data from the message	
            
      serverData = serverString.getBytes();
      urlData = url.getBytes();
      length1 = serverData.length;
      length2 = urlData.length;
            
      totalLength = length1 + length2 + 2;
      
      combinedData = new byte[totalLength];
      
      combinedData[0] = (byte) length1;      
            
      for (i = 0; i < length1; i ++) {
        
        combinedData[i + 1] = serverData[i];         
      }
      
      combinedData[length1 + 1] = (byte) length2;
      
      for (i = 0; i < length2; i ++) {
        
        combinedData[length1 + i + 2] = urlData[i];                 
      }                  
      
      m = new WatcherMessage(USER_CONNECTED, combinedData);
        
      Log.out.println("Watcher: connect message prepared: " + serverHost + " " + port + " " + url, 2);  
      
      if (output != null) {
      
        output.putMessage(m);        
        Log.out.println("Watcher: connect message sent", 2);
        
      }             
    }
    
    public void userDisconnected() {
      
      byte data[] = new byte[0];        
      WatcherMessage m = new WatcherMessage(USER_DISCONNECTED, data);  
      
      inTango = false;
                 
      Log.out.println("Watcher: disconnect message prepared", 2);
      
      if (output != null) {
      
        output.putMessage(m);
        Log.out.println("Watcher: disconnect message sent", 2);
        
      }        
    }           
    
    public void shutdown() {
           
      Log.out.println("Watcher: shutdown", 4);  
      reconnectEnabled = false;  
      
      if (input != null) {             
      
         input.stop();
      }
      
    }    
        
    public void reconnect() {
        
        Socket s = null;                
        
        Log.out.println("Watcher: reconnecting ....", 4);
        
        while(s == null && reconnectEnabled) {
        
            try {            
	   		    if(Const.inNetscape()) 
			    {
				    netscape.security.PrivilegeManager.enablePrivilege("30Capabilities");
			    }
			    if(Const.inMSIE())
			    {
				    com.ms.security.PolicyEngine.assertPermission(com.ms.security.PermissionID.SYSTEM); 
			    }
                s = new Socket("localhost", 4403);        
                output = new WatchThreadOut(s);
                input = new WatchThreadIn(s, output, this);     
                if (inTango) {
                    userConnected(serverHost, port, url);
                }
            
            } catch (Exception e) {
            
                Log.out.println("Watcher: something is wrong with our bloody socket today, reconnection in 10s ", 2);
    	    
	            try {
	            Thread.currentThread().sleep(10000);
    	        
	            } catch(Exception ex) {
    	        
	            ex.printStackTrace(Log.err);   
	            }
    	        
                s = null;
                output = null;
                input = null;            
            }                
        } 
    }
}



class WatchThreadIn extends WatcherProtocol implements Runnable
{
    private static final String CL="WatchThreadIn";
  

    private WatchNet n;
    private WatchThreadOut out;
    private Thread th;   //reference to this thread 
    private boolean exit;  
    private Watcher parent;

    public WatchThreadIn(Socket s, WatchThreadOut outpar, Watcher parent)
    {     
      out = outpar;        
      n = new WatchNet(s);   
      th = new Thread(this, CL + ":" + hashCode());
      exit = false;
      this.parent = parent;
      th.start();
      Log.out.println("WatcherThreadIn: thread started", 2);
     }// AppThreadIn ________________________________________________________

 
    public void run()
    {
   
      boolean tmpExit;  
   
      while(!exit)
      {
	WatcherMessage m;
	try
	{
	  m=n.receive();
	}
	catch(IOException e)
	{	 
	  m=null;
	  Log.out.println(CL + ": connection closed: " + e.getMessage(), 2);	  
	  tmpExit = exit;
	  stop();
	  if (!tmpExit) {
	    Log.out.println(CL + ": something is wrong with our bloody socket today, reconnecting now ", 2);	    	    
	    reconnect();	    	    	    
	  } else {
	    
	    Log.out.println(CL + ": regular shutdown, no reconnection attempted ", 2);	    
	  }
	  break;	  	  
	}
	if(m!=null)
	{
	  handleWATCHEREVENT(m);
	}
	else
	{
	 Log.err.println(CL+": null message received!: ignoring", 2);
	}
      }
    }// run ________________________________________________________________

    
    public void stop()
    {
      //System.out.println(CL+".stop()");
      exit=true;
      out.stop();
    
    }// stop _______________________________________________________________

    public void reconnect() {
        
        parent.reconnect();
        
    }

}// AppThreadIn ////////////////////////////////////////////////////////////


class WatchThreadOut implements Runnable
{
    private static final String CL="WatchThreadOut";


    private MessageQueue q;
    private WatchNet n;
    private Thread th;
    private boolean exit;
    

    static int number=1;    //for testing only
    int thisno;  //for testing only

    private Thread pinger;


    public WatchThreadOut(Socket s)
    {
      super();    

      thisno=number;
      number++;

      n=new WatchNet(s);
      q=new MessageQueue();
      th=new Thread(this, "WatchThreadOut:" + hashCode());
      exit=false;
      th.start();

      pinger=new Thread(new WatchPinger(n,600000), "Pinger:" + hashCode());
      pinger.start();
    }// AppThreadOut _______________________________________________________


    public void run()
    {
      Log.out.println(CL+".run(): "+thisno, 2);
      while(!exit)
      {
	Message m=q.getMessage();
	try
	{
	  n.send(m);
          //System.out.println(CL+": "+thisno+" sending "+m);
	}
	catch(IOException e)
	{
	  Log.err.println(CL+": error sending message: "+e, 2);
	  e.printStackTrace(Log.err);
	}
      }
    }// run ________________________________________________________________


    public synchronized void putMessage(Message m)
    {
      q.putMessage(m);
    }// putMessage _________________________________________________________

 
    public void stop()
    {
      //System.out.println(CL+".stop()");
      exit = true;
      q.unblock();
      pinger.stop();
      n.close();
    }// stop _______________________________________________________________


}




class WatchPinger implements Runnable
{
    private static final String CL="Pinger";
    
    private Thread _th;
    private int _interval;
    private WatchNet n;
    private boolean exit;


    public WatchPinger(WatchNet npar,int time)
    {
      n=npar;
      _interval=time;
      exit = false;
    }// Pinger ____________________________________________________________
    

    public void run()
    {
      byte[] data=new byte[0];     
      WatcherMessage pm=new WatcherMessage(Watcher.PING, data);

      //System.out.println(CL+".run()");
      while(!exit)
      {
	try
	{
	  Thread.sleep(_interval);
	}
	catch(InterruptedException e)
	{
	  Log.err.println(CL+": interruption: "+e, 2);
	  e.printStackTrace(Log.err);
	}
	try
	{
	    
	  if (!exit) {   
	    n.send(pm);
	  }  
	  //System.out.println(CL+": sent");
	}
	catch(IOException e)
	{
	  Log.err.println(CL+": error sending message: "+e, 2);
	  e.printStackTrace(Log.err);
	}
      }
    }// run ________________________________________________________________


    public void stop() {
        
        exit = true;           
        
    }

}// Pinger /////////////////////////////////////////////////////////////////


class WatcherProtocol
{
    private static final String CL="WatcherProtocol";        
      
    public void handleWATCHEREVENT(WatcherMessage m)
    {
      //lapps.sendAppToOther(AID,m);
      
      Log.out.println(CL + ": received message from PeopleWatch", 4);
      
    }// handleAPPEVENT _____________________________________________________


}// ApplicationProtocol ////////////////////////////////////////////////////