/***************************************************************************
  FILE: TLAgentScript.java, TomaszMajor-10Oct98
  PURPOSE: interface component for JavaScript
  $Id: TLAgentScript.java,v 1.8 1999/03/30 04:37:21 toma Exp $
  Component: Plugin, Interfaces
***************************************************************************/


package webwisdom.tango;


import java.util.Vector;

import webwisdom.tango.consts.CAConsts;
import webwisdom.tango.consts.Const;
import webwisdom.tango.messages.AppEventMessage;
import webwisdom.tango.structures.AppletHandle;
import webwisdom.tango.structures.LocalBase;
import webwisdom.tango.interfaces.AppletBaseInterf;

import netscape.javascript.JSObject;


/**Tango system proxy for scripts.
   This class may be used to create TAgent.
   *@see TAgent*/
public final class TLAgentScript
  implements TLAgent,AppletBaseInterf,TLAgentASType
{
    private static final String CL="TLAgentScript";


    private boolean inFrame=false;
    private JSObject script=null;
    private AppletHandle handle=null;
    private TLListener lowlList=null;


    /**Registers a JavaScript application in the Tango system.
     *@param app script to be registered
     *@exception TangoException if cannot get reference to Tango*/
    public TLAgentScript(JSObject scr) throws TangoException
    {
      if(scr==null)
	throw new IllegalArgumentException("null script");
      script=scr;
      
      try
      {
	handle=TLAgentApplet.register(this,script,this);
      }
      catch(UnsatisfiedLinkError e)
      {
	throw new TangoException("Not in Netscape");	
      }
      catch(NoClassDefFoundError e)
      {
	throw new TangoException("Not in Netscape");	
      }
      catch(TangoException e)
      {
	throw e;
      }
      if(handle==null)
	throw new TangoException("Tango demon not found");
    }// TLAgentScript ______________________________________________________


    public void exit()
    {
      //System.out.println(CL+".exit()");
      handle.remove();
      handle=null;
    }// exit _______________________________________________________________


    public void add(TLListener tl)
    {
      lowlList=tl;
    }// add ________________________________________________________________

  
    public final void send(AppEventMessage m)
    {
      handle.sendAppToOther(m);
    }// send _______________________________________________________________


    public void send(int userlist[],AppEventMessage m)
    {
      handle.sendAppToOther(userlist,m);
    }// send _______________________________________________________________


    public void receive(AppEventMessage m)
    {
      if(lowlList!=null)
	lowlList.receive(m);
    }// receive ____________________________________________________________


    public Object getJSWindow()
    {
      System.out.println(CL+".getJSWindow()");
      JSObject window=script;
      if(inFrame)
	return window;
      else
	return (JSObject)window.getMember("top");
//       try
//       {
//       //JSObject window = (JSObject)((JSObject)script.getWindow(scr));
// 	JSObject parent=(JSObject)script.getMember("top");
// 	String winName=(String)parent.getMember("name");
// 	JSObject targetWindow;
// 	System.out.println(CL + ": top window name " + winName);
// 	if(winName.equals("TangoCa"))
// 	{
// 	  System.out.println(CL + ": applet running in frame");
// 	  targetWindow = script;
// 	}
// 	else
// 	{
// 	  System.out.println(CL + ": applet running in window");
// 	  targetWindow = parent;
// 	}
// 	return targetWindow;  
//       }
//       catch(Exception e)
//       {
// 	return null;
//       }
    }// getJSWindow ________________________________________________________


    public void setInFrame(boolean i)
    {
      inFrame=i;
    }// inFrame ____________________________________________________________


    public String toString()
    {
      return CL+"[s="+script+",inF="+inFrame+",h="+handle+"]";
    }// toString ___________________________________________________________


}// TLAgentScript //////////////////////////////////////////////////////////

  
