/***************************************************************************
  FILE: TLAgentApplet.java, TomaszMajor,PiotrSokolowski-10Apr98
  PURPOSE: interface component for applets
  $Id: TLAgentApplet.java,v 1.15 1999/04/14 20:16:02 toma Exp $
  Component: Plugin, Interfaces
***************************************************************************/


package webwisdom.tango;


import java.applet.Applet;
import java.util.Vector;
import java.util.StringTokenizer;
import java.awt.*;

import webwisdom.tango.consts.CAConsts;
import webwisdom.tango.consts.Const;
import webwisdom.tango.messages.AppEventMessage;
import webwisdom.tango.structures.AppletHandle;
import webwisdom.tango.structures.LocalBase;
import webwisdom.tango.interfaces.AppletBaseInterf;

import netscape.javascript.JSObject;


/**Translates general Handler interface to lowlList.receive().*/
final class AemMsgHandler implements Handler
{


    private TLListener lowlList=null;


    public AemMsgHandler()
    {
    }// AemMsgHandler ______________________________________________________


    public synchronized void setListener(TLListener l)
    {
      lowlList=l;
    }// setListener ________________________________________________________

    
    public synchronized void handle(Object o)
    {
      if(lowlList!=null)
	lowlList.receive((AppEventMessage)o);
    }// handle _____________________________________________________________


}// AemMsgHandler //////////////////////////////////////////////////////////




/**Tango system proxy for applets.
   This class may be used to create TAgent.
   *@see TAgent*/
public final class TLAgentApplet
  implements TLAgent,AppletBaseInterf,TLAgentASType
{
    private static final String CL="TLAgentApplet";

    
    private boolean inFrame=false;
    private Applet applet=null;
    private AppletHandle handle=null;
    private TLListener lowlList;
    private AemMsgHandler aemMsgHandler;
    private QueueHandler msgQueue;
 

    /**Registers an applet in the Tango system.
     *@param app applet to be registered
     *@exception TangoException if cannot get reference to Tango*/
    public TLAgentApplet(Applet app) throws TangoException
    {
      if(app==null)
	throw new IllegalArgumentException("null applet");
      applet=app;


      String name=CL+"["+this+"].msgQueue";
      aemMsgHandler=new AemMsgHandler();
      msgQueue=new QueueHandler(name,aemMsgHandler);


      try
      {
	handle=register(this,(JSObject)JSObject.getWindow(applet),this);
      }
      catch(UnsatisfiedLinkError e)
      {
	throw new TangoException("Not in Netscape");	
      }
      catch(NoClassDefFoundError e)
      {
	throw new TangoException("Not in Netscape");	
      }
      catch(TangoException e)
      {
	throw e;
      }
      if(handle==null)
	throw new TangoException("Tango demon not found");

//       setOnUnloadHandler(applet);
    }// TLAgentApplet ______________________________________________________


//     private void setOnUnloadHandler(Applet app)
//     {
//       //System.out.println(CL+".setOnUnloadHandler()");
//       JSObject win=(JSObject)JSObject.getWindow(app);
//       Object e=win.eval("var Tango_onUnloadProcessor=null;");
//       e=win.eval("function Tango_setOnUnloadProcessor(p){Tango_onUnloadProcessor=p;}");
//       {
// 	Object args[]=new Object[1];
// 	args[0]=this;
// 	e=win.call("Tango_setOnUnloadProcessor",args);
//       }
//       e=win.eval("function Tango_onUnloadHandler(){Tango_onUnloadProcessor.onUnloadHandlerJS();}");
//       e=win.eval("window.onunload=Tango_onUnloadHandler;");
//       //System.out.println(CL+".setOnUnloadHandler(): eval="+eval);
//     }// setOnUnloadHandler _________________________________________________


//     public void onUnloadHandlerJS()
//     {
//       //System.out.println(CL+".onUnloadHandlerJS()");
//       exit();
//     }// onUnloadHandlerJS __________________________________________________


    public void exit()
    {
      //System.out.println(CL+".exit()");
      if(handle!=null)
      {
	handle.remove();
	handle=null;
      }
      msgQueue.stop();
    }// exit _______________________________________________________________


    public void add(TLListener tl)
    {
      lowlList=tl;
      aemMsgHandler.setListener(lowlList);
      if(!msgQueue.isRunning())
	(new Thread(msgQueue,msgQueue.getName())).start();
    }// add ________________________________________________________________


    public final void send(AppEventMessage m)
    {
      handle.sendAppToOther(m);
    }// send _______________________________________________________________


    public void send(int userlist[],AppEventMessage m)
    {
      handle.sendAppToOther(userlist,m);
    }// send _______________________________________________________________


    public void receive(AppEventMessage m)
    {
      System.out.println(CL+".receive("+m+")");
//       if(lowlList!=null)
// 	lowlList.receive(m);
      msgQueue.add(m);
    }// receive ____________________________________________________________


    public Object getJSWindow()
    {
//       if(!Const.inNetscape())
//       {
// 	Component c=applet;
// 	while(c!=null && !(c instanceof Frame))
// 	  c=c.getParent();
// 	return c;
//       }
      try
      {
	JSObject window=(JSObject)JSObject.getWindow(applet);
// 	JSObject parent=(JSObject)window.getMember("top");
// 	String winName=(String)parent.getMember("name");
// 	JSObject targetWindow=null;
// 	System.out.println(CL + ": top window name " + winName);
// 	if (winName.equals("TangoCa"))
// 	{
// 	  System.out.println(CL + ": applet running in frame");
// 	  //   name=(String)window.getMember("name");	
// 	  targetWindow = window;
// 	  // 	    inFrame = true;
// 	}
// 	else
// 	{
// 	  System.out.println(CL + ": applet running in window");
// 	  targetWindow = parent;
// 	  //    inFrame = false;	  
// 	}
// 	return targetWindow;
	if(inFrame)
	  return window;
	else
	  return (JSObject)window.getMember("top");
      }
      catch(Exception e)
      {
	e.printStackTrace();
	return null;
      }
    }// getJSWindow ________________________________________________________
    

    public void setInFrame(boolean i)
    {
      inFrame=i;
    }// inFrame ____________________________________________________________


//     static AppletHandle register(AppletBaseInterf base) throws TangoException
//     {
//       String name = null;  //name of the window or frame that contains register params
//       boolean inFrame = false;
//       String winName;
//       int AID,SID,AT;  

//       if(Const.inNetscape())
//       {
// 	netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserAccess");
// 	JSObject window=(JSObject) base.getJSWindow();;
// 	if(window==null)
// 	  return null;
// 	JSObject parent=(JSObject)window.getMember("top");  		 	
// 	winName=(String)parent.getMember("name");
// 	name = (String)window.getMember("name");	
//       }
//       else
//       {
// 	Object w=base.getJSWindow();
// 	if(w==null)
// 	  throw new TangoException("window not found");
// 	name=w.toString();
// 	winName="";
//       }
      
//       System.out.println(CL + ".register(): top window name " + winName);
      
//       if(winName.equals("TangoCa"))
//       {
// 	System.out.println(CL + ".register(): applet running in frame");
// 	inFrame = true;
//       }
//       else
//       {
// 	System.out.println(CL + ".register(): applet running in window");
// 	inFrame = false;	  
//       } 
      
      
//       if((name==null)||(name.length()==0))
//       {
// 	System.err.println(CL+".register(): corrupted window name!");
// 	return null;
//       }
      
//       AID=0;
//       try
//       {
// 	int AIDend=name.indexOf('_');
// 	int SIDend=name.indexOf('_',AIDend+1);
// 	AID=Integer.parseInt(name.substring(3,AIDend));
// 	SID=Integer.parseInt(name.substring(AIDend+1,SIDend));
// 	AT=Integer.parseInt(name.substring(SIDend+1));
	
// 	System.out.println(CL+".register(): AID="+AID+",SID="+SID+",AT="+AT);
//       }
//       catch(Exception e)
//       {
// 	System.err.println(CL+".register(): corrupted window name!: "+e);
// 	return null;
//       }
      
//       try
//       {
// 	AppletHandle h;  
	
// 	if (inFrame)
// 	  h=LocalBase.getLocalBase().reg.registerFrameApplet(base,AID,SID,AT, name);
// 	else
// 	  h=LocalBase.getLocalBase().reg.registerApplet(base,AID,SID,AT);
	
// 	return h;
//       }
//       catch(Exception e)
//       {
//       e.printStackTrace();
// 	System.err.println(CL+".register(): plugin not found!");     
// 	return null;
//       }
      
//     }// register ___________________________________________________________
    
    
    static AppletHandle register(AppletBaseInterf base,JSObject win,
				 TLAgentASType ast)
      throws TangoException
    {
      System.out.println(CL+".register()");

//       if(!Const.inNetscape())
//       {
// 	Object w=base.getJSWindow();
// 	if(w==null)
// 	  throw new TangoException("window not found");
// 	name=w.toString();
// 	winName="";
//       }


      netscape.security.PrivilegeManager.enablePrivilege("UniversalBrowserAccess");
      //JSObject win=(JSObject)JSObject.getWindow(app);
      JSObject top=(JSObject)win.getMember("top");
      String winName=(String)win.getMember("name");	
      String topName=(String)top.getMember("name");	
      boolean inFr=!topName.startsWith("win");
      String name=inFr?winName:topName;
      ast.setInFrame(inFr);
      System.out.println(CL+".register(): winName="+winName+
			 ",topName="+topName+",inFr="+inFr);

      int aid,sid,at;
      try
      {
	StringTokenizer st=new StringTokenizer(name.substring(3),"_");
	aid=Integer.valueOf(st.nextToken()).intValue();
	sid=Integer.valueOf(st.nextToken()).intValue();
	at=Integer.valueOf(st.nextToken()).intValue();
	//inFrame=(st.hasMoreTokens()&&st.nextToken().equals("f"));
	System.out.println(CL+".register(): aid="+aid+",sid="+sid+",at="+at);
      }
      catch(Exception e)
      {
	System.err.println(CL+".register(): corrupted window name!: "+e);
	e.printStackTrace();
	return null;
      }

      try
      {
	AppletHandle h;  
	if(inFr)
	  h=LocalBase.getLocalBase().reg.registerFrameApplet(base,aid,sid,at,name);
	else
	  h=LocalBase.getLocalBase().reg.registerApplet(base,aid,sid,at);
	return h;
      }
      catch(Exception e)
      {
	System.err.println(CL+".register(): plugin not found!");     
	e.printStackTrace();
	return null;
      }
    }// register ___________________________________________________________
    
    
    public String toString()
    {
      return CL+"[a=Applet@"+applet.hashCode()+
	",inF="+inFrame+",h="+handle+"]";
    }// toString ___________________________________________________________


}// TLAgentApplet //////////////////////////////////////////////////////////

  




