// $Id: AppSetMessage.java,v 1.1.2.2 1999/12/21 21:51:30 beca Exp $
/*
File: AppSetMessage.java
Component: Server, Plugin
Date		Author		Changes
Aug 22 96	Lukasz Beca	Created
*/


package webwisdom.tango.messages;


import webwisdom.tango.messages.Message;
import webwisdom.tango.consts.Const;
import webwisdom.tango.test.Log;
import java.io.*;  
import java.util.*;


/**
 * A class representing APPSET message
 * 
 *@author Lukasz Beca
 *@see Central Server - Local Demon protocol
 **/
public class AppSetMessage extends Message
{
    private static final String CL="AppSetMessage";
    
    private int UID;
    private Vector set;


/**
   * Creates a new SUPD message
   *@param PARAMparam update type: Const.ADDUPDATE, Const.REMOVEUPDATE, Const.MASTERUPDATE, Const.LOGINUPDATE, Const.LOGOUTUPDATE
   *@param SIDparam session number
   *@param ATpar application type
   *@param UIDparam user number
   *@param namepar user name
   **/

  public AppSetMessage(int UIDparam, Vector setparam) {
   
   UID = UIDparam;
   set = (Vector) setparam.clone();
   
   type = Const.APPSET;
  }

 /**
   * Creates a new SUPD message, information is read from the input stream
   *@param instream  input stream
   **/

  AppSetMessage(DataInputStream instream) throws IOException {
             
    ObjectInputStream ois;
    ByteArrayInputStream ais; 
    int len;
    byte table[];
    
    try {
    
        UID = instream.readInt();
        len = instream.readInt();
        table = new byte[len];
        instream.readFully(table);
        ais = new ByteArrayInputStream(table);
        ois = new ObjectInputStream(ais);
        set = (Vector) ois.readObject();
        type = Const.APPSET;        
        
    } catch(ClassNotFoundException e) {
        
        Log.err.println(e.getMessage(), 2);   
    }    
  }

 /**
   * Writes SUPD message to the output stream
   *@param outstream  output stream
   **/


  void send(DataOutputStream outstream) throws IOException {
     
    ByteArrayOutputStream aos = new ByteArrayOutputStream();
    ObjectOutputStream oos = new ObjectOutputStream(aos); 
    oos.writeObject(set);    
    
    byte table[] = aos.toByteArray();
     
    outstream.writeInt(Const.APPSET); 
    outstream.writeInt(UID);
    outstream.writeInt(table.length);
    outstream.write(table);
    
    outstream.flush();               
  }
  

 /**
   * Returns user number
   **/  


  public int getUID() {
       
    return UID;

  }  

 /**
   * Returns session number
   **/        

  public Vector getSet() {
   
    return set;
  }
  
  public String toString()
    {
      return CL+"[UID="+UID+",set="+set+"]";
    }// toString __________________________________________________________


}// SupdMessage ///////////////////////////////////////////////////////////


